/*
 * File:   main.c
 * Author: Renato Ferrero
 *
 * Created on 09 febbraio 2014, 17.37
 */

#include <xc.h>
#include <stdio.h>
#include <stdlib.h>
#include "PIC18F4550_config.h"

#define LCD_DEFAULT
#include "LCD_44780.h"
#include "LCD_44780.c"

#include <delay.h>
#include <delay.c>
#include<usart.h>



#define Cicalino LATCbits.LATC0  // Uscita cicalino
#define KeyGsmOn LATEbits.LATE0  // uscita per on/off modulo Gsm
#define LedVerde LATEbits.LATE1
#define LedRosso LATEbits.LATE2
#define PulsanteBT1 PORTBbits.RB4 // Pulsante BT1
#define PulsanteBT2 PORTBbits.RB5 // Pulsante BT2
unsigned char Beep(int, unsigned char); // funzione per cicalino
//****************FUNZIONI PER GESTIONE MODULO GSM
unsigned char VerificaGsmOnOff(void);
unsigned char ImpostaGsm(void);
unsigned char AzzeraDataUsart(unsigned char, unsigned char);
unsigned char ConfrontaDataUsart(char, char*);
unsigned char AttendiRispostaUSART(unsigned int);
unsigned char InviaSms(unsigned char, unsigned char);
unsigned char GestioneSMS(void);
unsigned char GestioneSquilli(void);
unsigned char InviaSquillo(void);
unsigned char InviaStringaUSART(char*);

// array per gestione modulo Gsm
unsigned char DataUsart [160]; // buffer dati ricevuti dal modulo gsm
unsigned char DataUsartOld; // memorizza ultimo carattere ricevuto
static char FlagDataUsart; // Memorizza l'avvenuta ricezione di dati dal modulo GSM
unsigned char CarAtt = 10; // memorizza il secondo carattere che si attende dal modulo
unsigned char CarPrec = 13; // memorizza il primo carattere che si attende dal modulo GSM
unsigned int TimOut; // usato per conteggiare il tempo massimo da AttendiRispostaUSART()
unsigned char Rx; // usato per conteggi nell'interrupt
unsigned char TipoGestore = 0; // 1=telecom   2=vodafone 3=Wind
unsigned char ErroreGsm = 0; // va ad uno se il setup modulo GSM non va a buon fine
unsigned char ContTimSquilli = 0; // conteggia il tempo massimo tra uno squillo e l'altro
unsigned char ContNumSquilli = 0; // Conteggia gli squilli in arrivo
unsigned char NumSquilli = 3; // Numero di squilli prima di attivare il comando

// inserire qui il numero di telefono utente , abilitato ad eseguire i comandi
unsigned char NumeroUtente [13] = {'+', '3', '9', '3', '3', '3', '7', '5', '5', '5', '4', '9', '9'};

unsigned char FlagSquilloOk = 0; // Se = 1 lo squillo  pervenuto dal numero Utente
unsigned char NumChiamante[13]; // memorizza il numero di chia ha chiamato
unsigned char NumSmsRicevuto; // memorizza il numero che identifica il SMS ricevuto
unsigned char FlagContrNumero = 0; // se = 0 non viene controllato il numero del chiamante--controllato da BT1
unsigned char FlagInviaSMS = 0; // se = 0 non vengono inviati SMS-- controllato da BT2

unsigned char x = 0;
unsigned char y = 0;

void main(void) {

    LATA = 0x00;
    TRISA = 0b11111111; // imposta pin come output=0 ...input=1
  // Imposto PORTB tutti input
    PORTB = 0x00;
    TRISB = 0b11111111;
  // Imposto PORTC tutti output
    LATC = 0x00;
    TRISC = 0b00000000;
    // Imposto PORTD tutti output
    LATD = 0x00;
    TRISD = 0b00000001;
    // Imposto PORTE tutti output
    LATE = 0b00000000;
    TRISE = 0b00000000;

    KeyGsmOn = 1;
    LedVerde = 1;
    LedRosso = 1;
    // Apro la porta usart
    OpenUSART(USART_TX_INT_OFF &
            USART_RX_INT_ON &
            USART_ASYNCH_MODE &
            USART_EIGHT_BIT &
            USART_CONT_RX &
            USART_BRGH_HIGH, 129); //(baud 9600=129   115200=10)

  
    // Abilito modalita' interruzione a due livelli alta e bassa
    RCONbits.IPEN = 1;

    // Abilito gli interrupt ad alta priorita'
    INTCONbits.GIEH = 1;

    // Abilito gli interrupt a bassa priorita'
    //INTCONbits.GIEL = 1;

    // Abilito interrupt per periferiche
    INTCONbits.PEIE = 1;
    INTCONbits.GIE = 1; // attivo
    INTCON2bits.RBPU = 0; // abilito i resistori di pull-up su PORTB

    OpenLCD(20);
    delay_ms(200);
    write_message_LCD("   FREEDOM-GSM");
    Beep(100, 1);
    backlight_LCD(TURN_ON_LED_LCD);
    delay_s(1);

    LedRosso = 0;
    LedVerde = 0;
    ClearLCD();
    VerificaGsmOnOff();
    ImpostaGsm();
    Beep(20, 3);
    delay_s(1);
    backlight_LCD(TURN_OFF_LED_LCD);

//*************************************************************
//***                      INIZIO CICLO INFINITO            ***
//*************************************************************
    while (1) {

        //*********************CONTROLLO OVERFLOW MODULO USART
        if (RCSTAbits.OERR) {
            Beep(100, 3);
            ClearLCD();
            WriteStringLCD("ERRORE USART GSM");
            goto_line_LCD(2);
            WriteStringLCD("BUFFER OVERFLOW!");
            delay_ms(2500);
            RCSTAbits.CREN = 0;
            RCSTAbits.CREN = 1;
        }

        //  ************ CONTROLLA SE E' IL BUFFER USART E' CARICO
        if (FlagDataUsart == 1) {

            // ************CONTROLLA SE E' ARRIVATO UN SMS
            if (ConfrontaDataUsart(2, (char*) "+CMTI") == 0) {
                GestioneSMS();
            }

            //   **********CONTROLLA SE E' IN ARRIVO UNA CHIAMATA ED ESEGUE PROGRAMMA CHIAMATE
            if (ConfrontaDataUsart(2, (char*) "RING") == 0) {
                delay_ms(100);
                if (ConfrontaDataUsart(11, (char*) "CLIP") == 0) {
                    // se il comando  +CLIP  in arrivo uno squillo
                    GestioneSquilli();
                }
            }

            AzzeraDataUsart(13, 10);
            FlagDataUsart == 0;
        }
        //************FINE CONTROLLO GSM

        // VERIFICA SE PASSA TROPPO TEMPO TRA DUE SQUILLI CONSECUTIVI,
        // SE SI SUPERA IL TEMPO MASSIMO SIGNIFICA CHE LA CHIAMATA E'
        // STATA INTERROTTA PER CUI SI AZZERA IL CONTATORE DEGLI QUILLI RICEVUTI
        if (ContNumSquilli > 0) {
            delay_ms(10);
            ContTimSquilli++;
            if (ContTimSquilli > 220) {
                ContTimSquilli = 0;
            }
        }

        // CONTROLLA SE SI STA PREMENDO IL TASTO BT1
        if (PulsanteBT1 == 0) {
            delay_ms(20);
            if (PulsanteBT1 == 0) {
                FlagContrNumero = !FlagContrNumero;
                ClearLCD();
                Beep(50, 1);
                WriteStringLCD("CONTR. NUM.CHIAM");
                goto_line_LCD(2);
                if (FlagContrNumero) {
                    WriteStringLCD("  ABILITATO");
                } else {
                    WriteStringLCD("  DISABILITATO");
                }
                while (PulsanteBT1 == 0) {
                    // attende che si rilasci BT1
                }
                delay_s(1);

            }
            ClearLCD();
            WriteStringLCD("MODULO IN ATTESA");
            goto_line_LCD(2);
            WriteStringLCD("  DI COMANDI");
        }


          // CONTROLLA SE SI STA PREMENDO IL TASTO BT2
        if (PulsanteBT2 == 0) {
            delay_ms(20);
            if (PulsanteBT2 == 0) {
                FlagInviaSMS = !FlagInviaSMS;
                ClearLCD();
                Beep(50, 1);
                WriteStringLCD("  INVIO SMS");
                goto_line_LCD(2);
                if (FlagInviaSMS) {
                    WriteStringLCD("  ABILITATO");
                } else {
                    WriteStringLCD("  DISABILITATO");
                }
                while (PulsanteBT2 == 0) {
                    // attende che si rilasci BT2
                }
                delay_s(1);

            }
            ClearLCD();
            WriteStringLCD("MODULO IN ATTESA");
            goto_line_LCD(2);
            WriteStringLCD("  DI COMANDI");
        }

    }
}

//****************************************************
//    BEEP
//******************************************************

unsigned char Beep(int Durata, unsigned char NumVolte) {
    unsigned char a = 0;
    for (a = 0; a < NumVolte; a++) {
        Cicalino = 1;
        delay_ms(Durata);
        Cicalino = 0;
        delay_ms(100);
    }
    return (1);
}

//*************************************************************************
//*************************************************************************
//                                                                       **
//                             DA QUI IN POI FUNZIONI PER MODULO GSM     **
//                                                                       **
//*************************************************************************
//*************************************************************************
//*************************************************************
//               VERIFICA SE IL MODULO E' ACCESO O NO
//*************************************************************

unsigned char VerificaGsmOnOff(void) {
    unsigned char Secondi;
    ClearLCD();
    WriteStringLCD("  VERIFICO IL");
    goto_line_LCD(2);
    WriteStringLCD("  MODULO GSM   ");
    delay_s(1);
 
    for (x = 0; x < 2; x++) {
        delay_ms(100);
        putrsUSART((char *) "AT\r\n"); // Invio un comando generico per far
        // autoapprendere al modulo la velocit di trasmissione
        delay_ms(200);

        putrsUSART((char *) "ATE0\r\n"); //ATE0: Disabilita eco dei comandi
        delay_ms(200);
        AzzeraDataUsart(13, 10);
        putrsUSART((char *) "AT\r\n");
        AttendiRispostaUSART(10);
        if (ConfrontaDataUsart(2, (char*) "OK") == 0) { // sel la risposta  OK il modulo  acceso
            // e funziona correttamente
             x = 2;
            ClearLCD();
            WriteStringLCD("  IL MODULO GSM");
            goto_line_LCD(2);
            WriteStringLCD("    E' ACCESO   ");
            delay_s(2);
        } else { // se si  qui  perch il modulo  spento oppure  bloccato ... occorre riaccenderlo
            ClearLCD();
            WriteStringLCD("GSM NON RISPONDE");
            goto_line_LCD(2);
            WriteStringLCD("   LO ACCENDO!");
            KeyGsmOn = 0; // Preme pulsante on/off
            delay_s(2);
            KeyGsmOn = 1;
            ClearLCD();
            WriteStringLCD("  ATTENDO PER ");
            for (Secondi = 10; Secondi > 0; Secondi--) {
                goto_line_LCD(2);
                WriteStringLCD("  ");
                WriteIntLCD(Secondi, 2);
                WriteStringLCD(" SECONDI");
                delay_s(1);
            }

        }
    }
    putrsUSART((char *) "ATE0\r\n"); //ATE0: Eco dei comandi disabilitato
    delay_ms(250);
    return (0);
}
//*************************************************************
//               IMPOSTA SETUP MODULO GSM
//*************************************************************

unsigned char ImpostaGsm(void) {
    unsigned char NumSms;
    unsigned char Segnale;
    //*********************** abilita ID chiamante
    AzzeraDataUsart(13, 10);
    putrsUSART((char *) "AT+CLIP=1\r\n");
    AttendiRispostaUSART(10);
    if (ConfrontaDataUsart(2, (char*) "OK") != 0) {
        ErroreGsm = 1;
    }

    //******************* imposta gli sms come testo
    AzzeraDataUsart(13, 10);
    putrsUSART((char *) "AT+CMGF=1\r\n");
    AttendiRispostaUSART(10);
    if (ConfrontaDataUsart(2, (char*) "OK") != 0) {
        ErroreGsm = 1;
    }
    // ******************CANCELLA SMS N 1 - 3
    for (x = 1; x < 4; x++) {
        //    delay_ms(100);
        AzzeraDataUsart(13, 10);
        NumSms = (x + 48);
        InviaStringaUSART((char*) "AT+CMGD=");
        WriteUSART(NumSms);
        putrsUSART((char *) "\r\n");
        AttendiRispostaUSART(20);
        if (ConfrontaDataUsart(2, (char*) "OK") != 0) {
            ErroreGsm = 1;
        }
    }
    delay_ms(100);
    if (ErroreGsm == 0) {
        Beep(100, 1);
        ClearLCD();
        WriteStringLCD("IMPOSTAZIONI GSM");
        goto_line_LCD(2);
        WriteStringLCD("ESEGUITE.  OK  ");
        delay_s(2);
        ClearLCD();
        WriteStringLCD("MODULO PRESENTE:");
        goto_line_LCD(2);
        AzzeraDataUsart(13, 10);
        putrsUSART((char *) "AT+CGMM\r\n");
        AttendiRispostaUSART(100);
        for (y = 2; y < 17; y++) {
            if (DataUsart[y] > 15) {
                write_char_LCD(DataUsart[y]);
            }
        }
        delay_s(2);
        ClearLCD();
        WriteStringLCD("LIVELLO SEGNALE");
        goto_line_LCD(2);
        WriteStringLCD("GSM (MAX30) = ");
        AzzeraDataUsart(13, 10);
        putrsUSART((char *) "AT+CSQ\r\n");
        AttendiRispostaUSART(10);
        Segnale = 0;
        if (DataUsart[9] == ',') {
            Segnale = (DataUsart[8] - 48);
        } else {
            Segnale = (DataUsart[8] - 48)*10;
            Segnale = Segnale + (DataUsart[9] - 48);
        }
        WriteIntLCD(Segnale, 2);
        delay_s(2);
        ClearLCD();
        WriteStringLCD("RICHIEDO GESTORE");
        goto_line_LCD(2);
        WriteStringLCD("...ATTENDERE");
        AzzeraDataUsart(13, 10);
        putrsUSART((char *) "AT+COPS?\r\n"); //COPS : RICHIEDE NOME GESTORE
        AttendiRispostaUSART(350);
        TipoGestore = 0;
        goto_line_LCD(2);
        //         WriteStringLCD("");
        if (ConfrontaDataUsart(14, (char*) "TELECOM") != 1) {
            TipoGestore = 1;
            WriteStringLCD("TIM         ");
        }
        if (ConfrontaDataUsart(14, (char*) "vodafone") != 1) {
            TipoGestore = 2;
            WriteStringLCD("VODAFONE     ");
        }
        if (ConfrontaDataUsart(14, (char*) "Wind") != 1) {// non verificato
            TipoGestore = 3;
            WriteStringLCD("VODAFONE     ");
        }
        if (TipoGestore == 0) {
            WriteStringLCD("NON RICONOSCIUTO");
        }
        delay_ms(2000);
        ClearLCD();
        WriteStringLCD("MODULO IN ATTESA");
        goto_line_LCD(2);
        WriteStringLCD("  DI COMANDI");
    } else {
        Beep(100, 3);
        ClearLCD();
        WriteStringLCD("IMPOSTAZIONI GSM");
        goto_line_LCD(2);
        WriteStringLCD("NON CORRETTE !  ");
        delay_ms(2000);
    }
    return (1);
}
//*************************************************************
//               ATTENDI RISPOSTA BUFFER USART
//*************************************************************

unsigned char AttendiRispostaUSART(unsigned int Tempo) {
    TimOut = 0;
    while ((TimOut < Tempo) && (FlagDataUsart == 0)) {
        delay_ms(100);
        TimOut++;
    }
    return (0);
}
//*************************************************************
//            CONFRONTA STRINGA DA DataUsart
//*************************************************************

unsigned char ConfrontaDataUsart(char Inizio, char* str3) {
    unsigned char x5 = 0;
    while (str3[x5] != '\0') {
        if (DataUsart[x5 + Inizio] != str3[x5]) {
            return (1);
        }
        x5++;
    }
    return (0);
}
//*************************************************************
//            INVIA STRINGA SU USART
//*************************************************************

unsigned char InviaStringaUSART(char* StrUsart) {
    unsigned char x6 = 0;
    while (StrUsart[x6] != '\0') {
        WriteUSART(StrUsart[x6]);
        delay_ms(10);
        x6++;
    }
    return (0);
}
//*************************************************************
//            AZZERA BUFFER DataUsart INTERRUPT
//*************************************************************

unsigned char AzzeraDataUsart(unsigned char UltimoCar, unsigned char CarAttuale) {

    for (Rx = 0; Rx < 160; Rx++) {
        DataUsart[Rx] = 0;
    }
    Rx = 0;
    CarPrec = UltimoCar; // Penultimo carattere da confr. per fermare lettura Buffer usart
    CarAtt = CarAttuale; // Ultimo carattere da confr. per fermare lettura usart
    FlagDataUsart = 0;
    TimOut = 0;
    return (1);
}

//*************************************************************
//            GESTIONE SQUILLI
//*************************************************************

unsigned char GestioneSquilli(void) {
    ClearLCD();
    if (ContNumSquilli == 0) {
        WriteStringLCD("CHIAM. IN ARRIVO");
    } else {
        WriteStringLCD("SQUILLO: ");
        WriteIntLCD((ContNumSquilli + 1), 1);
    }
    goto_line_LCD(2);
    WriteStringLCD("DA:");
    FlagSquilloOk = 1;
    for (x = 0; x < 13; x++) {
        write_char_LCD(DataUsart[18 + x]);
        NumChiamante[x] = DataUsart[18 + x];
    }
    if (FlagContrNumero==1) { // verifica numero chiamante se abilitato
        for (x = 0; x < 13; x++) {
            if (NumeroUtente[x] != NumChiamante[x]) {
                // il numero non  abilitato....riattacco
                delay_s(1);
                putrsUSART((char*) "ATH\r\n"); //
                AzzeraDataUsart(13, 10);
                FlagDataUsart == 0;
                ClearLCD();
                WriteStringLCD("  NUMERO NON");
                goto_line_LCD(2);
                WriteStringLCD("  ABILITATO");
                InviaSms(10, 1);
                ClearLCD();
                WriteStringLCD("MODULO IN ATTESA");
                goto_line_LCD(2);
                WriteStringLCD("  DI COMANDI");
                ContNumSquilli = 0;
                FlagSquilloOk = 0;
                return(0);
            }
        }
    }
    Beep(40, 1);
    if (FlagSquilloOk == 1) {
        ContNumSquilli++;
    }
    if (ContNumSquilli >= (NumSquilli - LedVerde)) {
        delay_s(3); //ritardo per far sentire l'ultimo squillo al chiamante
        putrsUSART((char*) "ATH\r\n"); //riattacca comunicazione
        delay_ms(100);
        ClearLCD();
        backlight_LCD(TURN_ON_LED_LCD);
        WriteStringLCD("   CHIAMATA OK");
        goto_line_LCD(2);
        WriteStringLCD("  ESEGUO COMANDO");
        LedVerde = !(LedVerde);
        Beep(50, 5);
        delay_s(6);
        if (LedVerde == 1) {
            InviaSquillo();
        }
        ClearLCD();
        WriteStringLCD("MODULO IN ATTESA");
        goto_line_LCD(2);
        WriteStringLCD("  DI COMANDI");
        delay_s(1);
        backlight_LCD(TURN_OFF_LED_LCD);
        AzzeraDataUsart(13, 10);
        FlagDataUsart == 0;
                ContNumSquilli = 0;
    }
    return(1);
}
//*************************************************************
//            GESTIONE SMS
//*************************************************************

unsigned char GestioneSMS(void) {
    unsigned char CodiceOk = 0;
    unsigned char FlagChiamataDaGestore = 0;
    unsigned char xy;
    unsigned char r = 0;
    unsigned char TestoSMS[160];

    for (x = 0; x < 160; x++) {
        TestoSMS[x] = 32; // 32 = spazio vuoto
    }
    backlight_LCD(TURN_ON_LED_LCD);
    ClearLCD();
    HomeLCD();
    WriteStringLCD("SMS IN ARRIVO");
    delay_ms(100);
    NumSmsRicevuto = DataUsart[14];

    AzzeraDataUsart('K', 13); // si intercetta la fine della parola OK (K)seguito da \r della stringa
    // che il modulo invia dopo aver ricevuto il comando AT+CMGR


    InviaStringaUSART((char*) "AT+CMGR="); // invia comando per leggere l'sms ricevuto
    WriteUSART(NumSmsRicevuto);
    delay_ms(10);
    putrsUSART((char*) "\r\n");

    AttendiRispostaUSART(20);

    r = 0;
    xy = 0;


    while (r < 3) { // ricerca la terza "  per vedere dove  il numero chiamante
        if (DataUsart[xy] == '"') {
            r++;
        }
        xy++;
    }

    if (!(DataUsart[xy] == '+')) { // VERIFICA SE E' UN SMS INVIATO DAL GESTORE
        FlagChiamataDaGestore = 1;
    } else {
        // *********************************************RILEVA NUMERO CHIAMANTE
        goto_line_LCD(2);
        WriteStringLCD("DA:");
        for (r = 0; r < 13; r++) {
            NumChiamante[r] = DataUsart[xy + r];
            write_char_LCD(NumChiamante[r]);
            //            WriteIntLCD( (NumChiamante[r]-48), 1);
        }
    }
    if (FlagContrNumero == 1) {
        for (r = 0; r < 13; r++) {
            if (NumeroUtente[r] != NumChiamante[r]) {
                AzzeraDataUsart(13, 10);
                FlagDataUsart == 0;
                ClearLCD();
                WriteStringLCD("SMS DA NUMERO");
                goto_line_LCD(2);
                WriteStringLCD("NON ABILITATO");
                InviaSms(10, 1);
                InviaStringaUSART((char*) "AT+CMGD=");
                WriteUSART(NumSmsRicevuto);
                putrsUSART((char *) "\r\n");
                delay_ms(100);
                ClearLCD();
                WriteStringLCD("MODULO IN ATTESA");
                goto_line_LCD(2);
                WriteStringLCD("  DI COMANDI");
                ContNumSquilli = 0;
                FlagSquilloOk = 0;
                return (0);

            }
        }
    }
    for (r = 5; r < 160; r++) {
        if (DataUsart[r] == 10) {
            xy = (r + 1); // a = inizio testo sms
            break;
        }
    }
    delay_s(2);
    ClearLCD();

    for (r = 0; r < 160; r++) {
        if (DataUsart[xy + r] == 13) {
            break;
        }
        TestoSMS[r] = DataUsart[xy + r];
    }
    for (r = 0; r < 16; r++) {
        write_char_LCD(TestoSMS[r]);
    }
    goto_line_LCD(2);
    WriteStringLCD("ESEGUO COMANDO");
    for (r = 0; r < 3; r++) { // TRASFORMO IN MAIUSCOLO LE PRIME 3 CIFRE DEL MESSAGGIO
        if ((TestoSMS[r] > 96) && (TestoSMS[r] < 123)) {
            TestoSMS[r] = TestoSMS[r] - 32;
        }
    }
    if ((TestoSMS[0] == 'O') && (TestoSMS[1] == 'N')) {
        if (LedRosso == 1) {
            InviaSms(4, 2);
        } else {
            LedRosso = 1;
            CodiceOk = 1;
            InviaSms(1, 2);
        }
    }
    if ((TestoSMS[0] == 'O') && (TestoSMS[1] == 'F')&& (TestoSMS[2] == 'F')) {
        if (LedRosso == 0) {
            InviaSms(3, 2);
        } else {
            LedRosso = 0;
            CodiceOk = 1;
            InviaSms(2, 2);
        }
    }
     // ****************************************CANCELLA SMS DA MEMORIA SIM
    InviaStringaUSART((char*) "AT+CMGD=");
    WriteUSART(NumSmsRicevuto);
    putrsUSART((char *) "\r\n");
    delay_ms(100);
    if (CodiceOk == 0) {
        ClearLCD();
        WriteStringLCD("CODICE SMS NON");
        goto_line_LCD(2);
        WriteStringLCD(" RICONOSCIUTO");
    }
    Beep(50, 5);
    delay_s(5);
    ClearLCD();
    WriteStringLCD("MODULO IN ATTESA");
    goto_line_LCD(2);
    WriteStringLCD("  DI COMANDI");
    delay_s(2);

    backlight_LCD(TURN_OFF_LED_LCD);
    return (1);
}

//*******************************************************************
//   INVIA SMS
//*******************************************************************
unsigned char InviaSms(unsigned char TipoSms, unsigned char SceltaNum) {
    // SceltaNum = 1 = Chiama il numero utente
    // SceltaNum = 2 = Chiama il numero del chiamante
    // SceltaNum = 3 = Chiama il gestore .... non utilizzato in questo progetto

    delay_s(2);
    ClearLCD();
    backlight_LCD(TURN_ON_LED_LCD);

    WriteStringLCD("INVIO SMS ");
    WriteIntLCD(TipoSms, 2);
    goto_line_LCD(2);


    for (x = 0; x < 13; x++) {
        switch (SceltaNum) {
            case 1:
                write_char_LCD(NumeroUtente[x]);
                break;
            case 2:
                write_char_LCD(NumChiamante[x]);
                break;
        }
        delay_ms(10);
    }
    if (FlagInviaSMS == 0) {
        delay_s(2);
        ClearLCD();
        WriteStringLCD("  INVIO SMS ");
        goto_line_LCD(2);
        WriteStringLCD("NON ABILITATO ");
        delay_s(2);
        return (0);
    }
    InviaStringaUSART((char*) "AT+CMGS=\"");

    // numero da comporre
    for (x = 0; x < 13; x++) {
        switch (SceltaNum) {
            case 1:
                WriteUSART(NumeroUtente[x]);
                break;
            case 2:
                WriteUSART(NumChiamante[x]);
                break;
        }
        delay_ms(10);
    }

    delay_ms(100);


    InviaStringaUSART((char*) "\"\r\n");
    delay_ms(1200);
    switch (TipoSms) {
        case 1:
            putrsUSART((char*) "LED ROSSO ACCESO CON SMS");
            break;
        case 2:
            putrsUSART((char*) "LED ROSSO SPENTO CON SMS");
            break;
        case 3:
            putrsUSART((char*) "IL LED ROSSO ERA GIA' SPENTO");
            break;
        case 4:
            putrsUSART((char*) "IL LED ROSSO ERA GIA' ACCESO");
            break;
        case 10:
            putrsUSART((char*) "E' ARRIVATO UN SMS/SQUILLO\r\n");
            putrsUSART((char*) "DAL SEGUENTE NUMERO: \r\n");
            for (x = 0; x < 13; x++) {
                WriteUSART(NumChiamante[x]);
                delay_ms(10);
            }
            break;
    }

    delay_ms(100);

    AzzeraDataUsart(13, 10);
    WriteUSART(0x1A);

    AttendiRispostaUSART(100);
    ClearLCD();
    // controllo risposta ad invio messagio +CMGS
    // la risposta per la SIM_340D inizia dal carattere [2] di DataUsart mentre
    // per la SIM_900 inizia dalla posizione [5].
    if ((ConfrontaDataUsart(2, (char*) "+CMGS:") == 0) || (ConfrontaDataUsart(5, (char*) "+CMGS:") == 0)) {
        WriteStringLCD(" SMS INVIATO ");
    } else {
        WriteStringLCD("SMS NON INVIATO");
    }
    goto_line_LCD(2);
    WriteStringLCD("CORRETTAMENTE");
    delay_ms(100);
    AzzeraDataUsart(13, 10);
    delay_s(2);
    ClearLCD();
    WriteStringLCD("MODULO IN ATTESA");
    goto_line_LCD(2);
    WriteStringLCD("  DI COMANDI");
    return (0);
}
//************************************************************
//   INVIA SQUILLO
//************************************************************
unsigned char InviaSquillo(void) {
       ClearLCD();
       Beep(100,1);
        WriteStringLCD("INVIO SQUILLO DI");
        goto_line_LCD(2);
        WriteStringLCD("   CONFERMA");
            InviaStringaUSART((char*) "ATD");
    delay_ms(10);
    for (x = 0; x < 13; x++) {
        WriteUSART(NumChiamante[x]);
        delay_ms(10);
    }
    WriteUSART(';');
    delay_ms(10);
    WriteUSART('\r');
    delay_ms(10);
    WriteUSART('\n');
    delay_s(12);
    putrsUSART((char *) "ATH\r\n");
    return (0);
}
//*************************************************************
//           INTERRUPT PER RICEZIONE PORTA SERIALE (GSM)
//*************************************************************
__interrupt(high_priority) void RxUsart(void) {
    // Controllo che l'interrupt sia stato generato dall'USART
    if (PIR1bits.RCIF == 1) {
        DataUsartOld = DataUsart[Rx - 1];
        // Leggo il dato dal buffer di ricezione
        DataUsart[Rx] = ReadUSART();

        if (FlagDataUsart == 0) {
            if ((DataUsartOld == CarPrec && DataUsart[Rx] == CarAtt && Rx > 3)) {
                //                if (Rx > 3) { // esclude avanzamento riga (13 + 10)inviato dal
                // modulo gsm prima del comando
                FlagDataUsart = 1;
            }
        } else {
        }
        if (Rx < 159) {
            Rx++;
        } else {
            FlagDataUsart = 1;
        }
        PIR1bits.RCIF = 0;
    }
}
