/*
 * File:   main.c
 * Author: Renato Ferrero
 *
 * Created on 09 febbraio 2014, 17.37
 */

#include <xc.h>
#include <stdio.h>
#include <stdlib.h>
#include "PIC18F4550_config.h"

#define LCD_DEFAULT
#include <delay.h>
#include <delay.c>
#include <LCD_44780.h>
#include <LCD_44780.c>


#include<usart.h>



#define Cicalino LATCbits.LATC0  // Uscita cicalino
#define KeyGsmOn LATEbits.LATE0  // uscita per on/off modulo Gsm
#define LedVerde LATEbits.LATE1
#define LedRosso LATEbits.LATE2
#define PulsanteBT1 PORTBbits.RB4 // Pulsante BT1
#define PulsanteBT2 PORTBbits.RB5 // Pulsante BT2

unsigned char Beep(int, unsigned char); // funzione per cicalino
unsigned char AzzeraDataUsart(unsigned char, unsigned char);


// array per gestione modulo Gsm
unsigned char DataUsart [160]; // buffer dati ricevuti dal modulo gsm
unsigned char DataUsartOld; // memorizza ultimo carattere ricevuto
static char FlagDataUsart; // Memorizza l'avvenuta ricezione di dati dal modulo GSM
unsigned char CarAtt = 10; // memorizza il secondo carattere che si attende dal modulo
unsigned char CarPrec = 13; // memorizza il primo carattere che si attende dal modulo GSM
unsigned int TimOut; // usato per conteggiare il tempo massimo da AttendiRispostaUSART()
unsigned char Rx; // usato per conteggi nell'interrupt
unsigned char x;

void main(void) {


    LATA = 0x00;

    TRISA = 0b11111111; // imposta pin come output=0 ...input=1
    // imposta pin come output se 0 ...o come input se 1
    PORTB = 0x00;
    TRISB = 0b11111111;
    LATC = 0x00;
    TRISC = 0b00000000;
    // Imposto PORTD tutte uscite
    LATD = 0x00;
    TRISD = 0b00000001;
    // Imposto PORTE tutte uscite
    LATE = 0b00000000;
    TRISE = 0b00000000;
    KeyGsmOn = 1;
    LedVerde = 1;
    LedRosso = 1;

    //Apro porta USART a 9600 baud
    OpenUSART(USART_TX_INT_OFF &
            USART_RX_INT_ON &
            USART_ASYNCH_MODE &
            USART_EIGHT_BIT &
            USART_CONT_RX &
            USART_BRGH_HIGH, 129); //baud:  9600=129  115200=10

    INTCON2bits.RBPU = 0; // abilito i resistori di pull-up su PORTB
    //************************************************************
    // Abilito le interruzioni
    //************************************************************

    // Abilito modalita' interruzione a due livelli alta e bassa
    RCONbits.IPEN = 1;

    // Abilito gli interrupt ad alta priorita'
    INTCONbits.GIEH = 1;

    // Abilito gli interrupt a bassa priorita'
    //INTCONbits.GIEL = 1;

    // Abilito interrupt per periferiche
    INTCONbits.PEIE = 1;
    INTCONbits.GIE = 1; // attivo

    OpenLCD(20);
    delay_ms(200);
    write_message_LCD("PROGR. ESEMPIO");
    goto_line_LCD(2);
    WriteStringLCD("Test_Rx_Usart");
    Beep(50
            , 1);
    backlight_LCD(TURN_ON_LED_LCD);
    delay_s(1);
    LedRosso = 0;
    LedVerde = 0;
    ClearLCD();
    WriteStringLCD("DataRxUsart =");
    goto_line_LCD(2);
    Beep(50, 3);
    delay_s(1);


    putrsUSART((char*) "AT\r\n");
    delay_ms(300);
    putrsUSART((char*) "ATE0\r\n");
    delay_ms(500);
    backlight_LCD(TURN_OFF_LED_LCD);

    AzzeraDataUsart(13, 10);

    while (1) {

        //*********************CONTROLLO OVERFLOW MODULO USART
        if (RCSTAbits.OERR) {
              backlight_LCD(TURN_ON_LED_LCD);
            Beep(100, 3);
            ClearLCD();
            WriteStringLCD("ERRORE USART GSM");
            goto_line_LCD(2);
            WriteStringLCD("BUFFER OVERFLOW!");
            delay_ms(2500);
            RCSTAbits.CREN = 0;
            RCSTAbits.CREN = 1;
            ClearLCD();
            backlight_LCD(TURN_OFF_LED_LCD);
        }


        if (FlagDataUsart) { // La porta USART ha rilevato un input
            backlight_LCD(TURN_ON_LED_LCD);
            Beep(50, 2);
            for (x = 0; x < 16; x++) {
                if (DataUsart[x] > 31) {
                    write_char_LCD(DataUsart[x]);
                }
            }
            delay_s(3);
            Beep(50, 1);
            ClearLCD();
            WriteStringLCD("DataRxUsart =");
            goto_line_LCD(2);
            backlight_LCD(TURN_OFF_LED_LCD);
            AzzeraDataUsart(13, 10);
        }


                if (PulsanteBT1 == 0) {
            delay_ms(20);
            if (PulsanteBT1 == 0) {
                Beep(50, 1);
                putrsUSART((char *) "AT\r\n");

                while (PulsanteBT1 == 0) {
                    // attende che si rilasci BT1
                }

            }



        }
        if (PulsanteBT2 == 0) {
            delay_ms(20);
            if (PulsanteBT2 == 0) {

                Beep(50, 1);
                putrsUSART((char*) "AT+CGMM\r\n");

                while (PulsanteBT2 == 0) { // attende che si rilasci BT2
                }

            }



        }



    }
}

//****************************************************
//    BEEP
//******************************************************

unsigned char Beep(int Durata, unsigned char NumVolte) {
    unsigned char a = 0;
    for (a = 0; a < NumVolte; a++) {
        Cicalino = 1;
        delay_ms(Durata);
        Cicalino = 0;
        delay_ms(100);
    }
    return (1);
}
//*************************************************************
//            AZZERA BUFFER DataUsart INTERRUPT
//*************************************************************

unsigned char AzzeraDataUsart(unsigned char UltimoCar, unsigned char CarAttuale) {

    for (Rx = 0; Rx < 220; Rx++) {
        DataUsart[Rx] = 0;
    }
    Rx = 0;
    CarPrec = UltimoCar; // Penultimo carattere da confr. per fermare lettura Buffer usart
    CarAtt = CarAttuale; // Ultimo carattere da confr. per fermare lettura usart
    FlagDataUsart = 0;
    TimOut = 0;
    return (1);
}

//*************************************************************
//           INTERRUPT PER RICEZIONE PORTA SERIALE (GSM)
//*************************************************************
//void interrupt Interruzione(){

__interrupt(high_priority) void RxUsart(void) {
        if (PIR1bits.RCIF == 1) {// Controllo che l'interrupt sia stato generato dall'USART
        DataUsartOld = DataUsart[Rx - 1]; // Leggo il dato dal buffer di ricezione
        DataUsart[Rx] = ReadUSART();

        //*********delay_s(1) DA INSERIRE PER PROVA OVERFLOW....POI RICORDARSI DI ELIMINARLO !!
        //  delay_s(1);
        //*********************
        if (FlagDataUsart == 0) {
            if ((DataUsartOld == CarPrec && DataUsart[Rx] == CarAtt && Rx > 3)) {
                //                if (Rx > 3) { // esclude avanzamento riga (13 + 10)inviato dal
                // modulo gsm prima del comando
                FlagDataUsart = 1;
            }
        } else {

        }
        if (Rx < 16) {
            Rx++;
        } else {
            FlagDataUsart = 1;
        }
        PIR1bits.RCIF = 0;
    }



}

